<?php
// Verbindung zur Datenbank herstellen
require_once 'db_connect.php';

function getMappingFromDatabase($pdo) {
    $stmt = $pdo->prepare('SELECT value FROM settings WHERE key_name = "lagerplatz_mapping" LIMIT 1');
    $stmt->execute();
    $result = $stmt->fetchColumn();

    if ($result) {
        return json_decode($result, true); // Zuordnung als Array zurückgeben
    }

    // Wenn keine Zuordnung in der Datenbank vorhanden ist
    return [];
}

function getAvailableSpots($pdo, $art, $anzahl) {
    // Zuordnungen aus der Datenbank laden
    $mapping = getMappingFromDatabase($pdo);

    if (!isset($mapping[$art])) {
        return ['allocated' => [], 'remaining' => $anzahl]; // Keine Zuordnung gefunden
    }

    $ebene = $mapping[$art];

    // Dynamik-Daten abrufen
    $dynamikQuery = "
        SELECT lagerplatz_id, COUNT(*) AS wechsel
        FROM inventur_protokoll
        GROUP BY lagerplatz_id
        ORDER BY wechsel DESC
    ";
    $dynamikResult = $pdo->query($dynamikQuery)->fetchAll(PDO::FETCH_KEY_PAIR);

    $query = "
        SELECT id, name, kapazitaet, belegung 
        FROM lagerplaetze 
        WHERE name LIKE :ebene 
        AND (kapazitaet - belegung) > 0 
        ORDER BY name ASC
    ";

    $stmt = $pdo->prepare($query);
    $stmt->execute(['ebene' => $ebene . '%']);
    $lagerplaetze = $stmt->fetchAll();

    $result = [];
    $remaining = $anzahl;

    // Lagerplätze nach Priorität sortieren (Dynamik)
    usort($lagerplaetze, function ($a, $b) use ($dynamikResult) {
        $wechselA = $dynamikResult[$a['id']] ?? 0;
        $wechselB = $dynamikResult[$b['id']] ?? 0;

        return $wechselB <=> $wechselA; // Sortierung nach Häufigkeit des Wechsels
    });

    // Zuweisung basierend auf Kapazität
    foreach ($lagerplaetze as $platz) {
        $frei = $platz['kapazitaet'] - $platz['belegung'];

        if ($frei > 0) {
            $to_allocate = min($frei, $remaining);
            $result[] = [
                'id' => $platz['id'],
                'name' => $platz['name'],
                'allocated' => $to_allocate
            ];
            $remaining -= $to_allocate;

            if ($remaining <= 0) break;
        }
    }

    return ['allocated' => $result, 'remaining' => $remaining];
}

// Wenn das Formular per POST abgeschickt wird
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['action']) && $_POST['action'] === 'confirm') {
        // Lagerplätze aktualisieren
        $allocated = json_decode($_POST['allocated'], true);
        foreach ($allocated as $platz) {
            $updateQuery = "UPDATE lagerplaetze SET belegung = belegung + :allocated WHERE id = :id";
            $updateStmt = $pdo->prepare($updateQuery);
            $updateStmt->execute(['allocated' => $platz['allocated'], 'id' => $platz['id']]);
        }
        echo "<div class='alert alert-success'>Lagerplätze wurden erfolgreich gebucht!</div>";
        exit;
    }

    $anzahl = intval($_POST['anzahl'] ?? 0);
    $art = $_POST['art'] ?? '';

    if ($anzahl > 0 && !empty($art)) {
        $result = getAvailableSpots($pdo, $art, $anzahl);

        if ($result['remaining'] > 0) {
            echo "<div class='alert alert-danger'>Nicht genügend Platz für $anzahl Paletten. Es fehlen {$result['remaining']} Paletten.</div>";
        } else {
            echo "<div class='alert alert-success'>Lagerplätze erfolgreich zugewiesen.</div>";
        }

        echo "<h5>Zugewiesene Lagerplätze:</h5>";
        echo "<ul class='list-group' id='lagerplaetzeListe'>";
        foreach ($result['allocated'] as $platz) {
            echo "<li class='list-group-item' data-id='" . htmlspecialchars($platz['id']) . "'>
                    Lagerplatz: " . htmlspecialchars($platz['name']) . 
                    " – Zugeteilt: " . htmlspecialchars($platz['allocated']) . " Paletten
                  </li>";
        }
        echo "</ul>";

        // Buttons anzeigen
        echo "
        <form id='actionForm' class='mt-4'>
            <input type='hidden' name='allocated' value='" . json_encode($result['allocated']) . "'>
            <button type='button' id='confirmButton' class='btn btn-success'>Lagerplätze gebucht</button>
            <button type='button' id='cancelButton' class='btn btn-danger'>Aktion abgebrochen</button>
        </form>";
        exit;
    } else {
        echo "<div class='alert alert-danger'>Bitte geben Sie gültige Werte ein.</div>";
        exit;
    }
}
?>

<!-- Modal-Inhalt -->
<form id="sehenForm" method="POST">
    <div class="mb-3">
        <label for="anzahl" class="form-label">Anzahl der Paletten</label>
        <input type="number" id="anzahl" name="anzahl" class="form-control" min="1" required>
    </div>
    <div class="mb-3">
        <label for="art" class="form-label">Art der Ware</label>
        <select id="art" name="art" class="form-select" required>
            <option value="">Bitte wählen</option>
            <?php foreach (getMappingFromDatabase($pdo) as $key => $value): ?>
                <option value="<?php echo htmlspecialchars($key); ?>"><?php echo htmlspecialchars($key); ?></option>
            <?php endforeach; ?>
        </select>
    </div>
    <button type="submit" class="btn btn-primary">Berechnen</button>
</form>
<div id="sehenErgebnis" class="mt-3"></div>

<style>
    .list-group-item.erledigt {
        text-decoration: line-through;
        color: #888;
    }
</style>

<script>
document.addEventListener('DOMContentLoaded', function() {
    const form = document.getElementById('sehenForm');
    const ergebnisContainer = document.getElementById('sehenErgebnis');

    // Berechnungsformular
    form.addEventListener('submit', function(event) {
        event.preventDefault(); // Standard-Formular-Submit verhindern

        const formData = new FormData(form);

        // AJAX-Request senden
        fetch('sehen.php', {
            method: 'POST',
            body: formData,
        })
        .then(response => response.text())
        .then(result => {
            // Ergebnis anzeigen
            ergebnisContainer.innerHTML = result;

            // Liste der Lagerplätze durch Anklicken als erledigt markieren
            const liste = document.getElementById('lagerplaetzeListe');
            if (liste) {
                liste.addEventListener('click', function(event) {
                    if (event.target && event.target.classList.contains('list-group-item')) {
                        event.target.classList.toggle('erledigt');
                    }
                });
            }

            // Buttons aktivieren
            const confirmButton = document.getElementById('confirmButton');
            const cancelButton = document.getElementById('cancelButton');

            if (confirmButton) {
                confirmButton.addEventListener('click', function() {
                    const allocatedData = new FormData(document.getElementById('actionForm'));
                    allocatedData.append('action', 'confirm');

                    fetch('sehen.php', {
                        method: 'POST',
                        body: allocatedData,
                    })
                    .then(response => response.text())
                    .then(result => {
                        ergebnisContainer.innerHTML = result;
                    });
                });
            }

            if (cancelButton) {
                cancelButton.addEventListener('click', function() {
                    form.reset();
                    ergebnisContainer.innerHTML = '';
                });
            }
        })
        .catch(error => {
            console.error('Fehler:', error);
            ergebnisContainer.innerHTML = '<div class="alert alert-danger">Es ist ein Fehler aufgetreten.</div>';
        });
    });
});
</script>
